import psycopg2
from loguru import logger
from psycopg2 import extras

from gdshoplib.core.settings import DBSettings


class GDShop:
    # Класс для работы с данными из сервисов gdshop
    ...


class DB:
    def __init__(self, data=None):
        self.data = data
        self.settings = DBSettings()

    def execute(func):
        def wrap(self, *args, **kwargs):
            with psycopg2.connect(self.settings.DB_DSB) as conn:
                with conn.cursor(cursor_factory=extras.DictCursor) as curs:
                    query = func(self, *args, **kwargs)
                    try:
                        curs.execute(query)
                        data = curs.fetchall()
                        if not data:
                            logger.warning(f"Запись не найдена: {query}")
                            raise DBNotFound
                        return data
                    except psycopg2.Error as e:
                        logger.error(query)
                        logger.exception(e)
                        raise e

        return wrap

    @execute
    def get_price(self, sku):
        return f"select full_price, base_price, discount, profit from v1.prices p where sku = '{sku}';"

    @execute
    def get_cart(self, page_id):
        return f"select sku, name, url, source_price, sell_price, quantity, discount, supply_discount, error from v1.order_carts where id = '{page_id}'"

    @execute
    def get_partners_products(self):
        return "select sku from v1.partners_stock"

    @execute
    def get_market_products(self):
        return "select sku from v1.prices where quantity > 1"


class DBNotFound(Exception):
    ...



import re

from lxml import etree, objectify

from gdshoplib.apps.platforms.base import Platform
from gdshoplib.core.settings import ProductSettings
from gdshoplib.packages.feed import Feed
from gdshoplib.services.notion.database import Database


class YandexBussinesManager(Platform, Feed):
    KEY = "YAMB"
    DESCRIPTION_TEMPLATE = "yamb.txt"

    def get_categories(self):
        categories = Database(ProductSettings().CATEGORY_DB).pages()
        return [(str(i.uid), i.title) for i in categories]

    @staticmethod
    def fetch_id(id):
        return str(sum([int(i) for i in re.sub(r"\D*", "", id)]))

    @property
    def product_filter(self):
        return dict(status_description="Готово", status_publication="Публикация")

    def get_shop(self):
        shop = objectify.Element("shop")

        shop.name = self.feed_settings.SHOP_NAME
        shop.company = self.feed_settings.COMPANY_NAME
        shop.url = self.feed_settings.SHOP_URL

        currencies = objectify.Element("currencies")
        currency = etree.Element("currency")
        currency.attrib["id"] = "RUB"
        currency.attrib["rate"] = "1"
        objectify.deannotate(currency, cleanup_namespaces=True, xsi_nil=True)
        objectify.deannotate(currencies, cleanup_namespaces=True, xsi_nil=True)
        currencies.append(currency)
        shop.currencies = currencies

        categories = objectify.Element("categories")
        for ind, k in enumerate(
            ["Все товары", "Спорт и отдых", "Конный спорт"], start=1
        ):
            category = etree.Element("category")
            category.attrib["id"] = str(ind)
            if ind - 1 != 0:
                category.attrib["parentId"] = str(ind - 1)
            category.text = k
            categories.append(category)

        for _id, name in self.get_categories():
            category = etree.Element("category")
            category.attrib["id"] = _id
            category.attrib["parentId"] = "3"
            category.text = name
            objectify.deannotate(category, cleanup_namespaces=True, xsi_nil=True)
            categories.append(category)

        shop.categories = categories
        objectify.deannotate(shop, cleanup_namespaces=True, xsi_nil=True)

        return shop

    def get_offer(self, product):
        appt = objectify.Element("offer")
        appt.attrib["id"] = product.sku
        appt.count = product.quantity
        appt.weight = product.weight
        appt.dimensions = product.dimensions
        if product.quantity == 0:
            appt.attrib["type"] = "on.demand"

        appt.currencyId = "RUB"
        appt.price = product.price.now
        if ((product.price.current_discount / product.price.base_price) * 100) > 5:
            appt.oldprice = product.price.base_price

        appt.purchase_price = product.price.neitral
        appt.cofinance_price = product.price.neitral

        for image in product.images:
            appt.addattr("picture", image.get_url())

        if product.color:
            appt.addattr("Цвет", product.color)
        if product.size:
            appt.addattr("Размер", product.size)

        appt.name = product.name
        appt.description = product.description.generate(self)
        appt.vendor = product.brand.title
        appt.categoryId = str(product.categories[0].uid)

        objectify.deannotate(appt, cleanup_namespaces=True, xsi_nil=True)
        return appt



from gdshoplib.core.settings import PriceSettins
from gdshoplib.services.gdshop.gdshop import DB

price_settings = PriceSettins()


class Price:
    def __init__(self, product):
        self.product = product
        self._db_request = None

    @property
    def raw(self):
        if not self._db_request:
            self._db_request = DB().get_price(self.product.sku)[0]
        return self._db_request

    @property
    def current_discount(self):
        return int(self.raw["discount"])
        # TODO
        # Получить текущую скидку
        return 100 - round(self.now / self.profit * 100)

    @property
    def base_price(self):
        return int(self.raw["full_price"]) + int(self.raw["discount"])

    @property
    def now(self):
        return int(self.raw["full_price"])
        discount = self.get_score()
        if discount:
            _now = self.profit + self.profit * (discount * 0.01)
            if _now < self.neitral:
                return self.neitral
            return _now

        return self.profit

    @property
    def eur(self):
        # Цена в EUR
        return self.product.price_eur

    @property
    def net(self):
        # Цена в рублях
        return self.eur * price_settings.EURO_PRICE

    @property
    def gross(self):
        # Цена с учетом расходов и налогов на закупку
        return int(self.raw["base_price"])

    @property
    def neitral(self):
        # Цена безубыточности
        return int(self.raw["base_price"])

    @property
    def profit(self):
        return int(self.raw["profit"])

    def __getitem__(self, key):
        try:
            return super(Price, self).__getattribute__(key)
        except AttributeError:
            return self.__getattr__(key)



from typing import Optional

import boto3
from botocore.exceptions import ClientError
from pydantic import BaseModel

from gdshoplib.core.settings import S3Settings


class File(BaseModel):
    key: str
    content: Optional[bytes]
    mime: Optional[str]


class S3v2:
    _client = None

    def __init__(self, bucket="gdshop"):
        self.bucket = bucket
        self.content = None

    @classmethod
    @property
    def client(cls):
        if not cls._client:
            settings = S3Settings()
            cls._client = boto3.session.Session().client(
                service_name="s3",
                endpoint_url=settings.ENDPOINT_URL,
                aws_access_key_id=settings.ACCESS_KEY,
                aws_secret_access_key=settings.SECRET_KEY,
            )
        return cls._client

    def exception_handle(func):
        def wrap(*args, **kwargs):
            try:
                r = func(*args, **kwargs)
                return r["ResponseMetadata"]["HTTPStatusCode"] == 200
            except (ClientError, KeyError):
                return None

        return wrap

    @exception_handle
    def put(self, object: File):
        return self.client.put_object(
            Bucket=self.bucket,
            Key=object.key,
            Body=object.content,
            ACL="public-read",
            StorageClass="ICE",
            ContentType=object.mime,
        )

    @exception_handle
    def exists(self, object: File):
        return self.__class__.client.head_object(Bucket=self.bucket, Key=object.key)

    @exception_handle
    def get(self, object: File):
        return self.__class__.client.get_object(Bucket=self.bucket, Key=object.key)

    @exception_handle
    def delete(self, object: File):
        return self.__class__.client.delete_object(Bucket=self.bucket, Key=object.key)

    def __str__(self) -> str:
        return f"{self.__class__}: {self.bucket}"

    def __repr__(self) -> str:
        return f"{self.__class__}: {self.bucket}"



import time
from typing import Optional

import typer

from gdshoplib.packages.feed import Feed

app = typer.Typer()


@app.command()
def warm(platform_key=None, loop_iteration: Optional[int] = typer.Option(None)):
    while True:
        if platform_key:
            warm_platfrom_feed(platform_key)
            return

        for platform in [Feed, *[feed for feed in Feed.__subclasses__()]]:
            warm_platfrom_feed(platform.KEY)

        if loop_iteration:
            time.sleep(loop_iteration)
        else:
            break


###


def warm_platfrom_feed(key):
    platform = Feed.get_platform_class(key=key)
    platform().push_feed()
    print(platform)



import re

import backoff
from PIL import ImageDraw, ImageFont

import requests
from gdshoplib.core.settings import GeneralSettings
from gdshoplib.packages.renderer import ImageRenderer
from gdshoplib.packages.s3 import S3
from gdshoplib.services.notion.block import Block
from gdshoplib.services.notion.page import Page


class ProductMedia(Block):
    def __init__(self, *args, **kwargs):
        super(ProductMedia, self).__init__(*args, **kwargs)
        self.response = None
        self.s3 = S3(self)

    @property
    def url(self):
        return self[self.type]["file"]["url"]

    @property
    def badges(self):
        result = []
        for badge in self.parent.badges:
            if not badge.work or not badge.file:
                continue
            result.append(badge)
        return result

    @property
    def key(self):
        return self.notion.get_capture(self) or f"{self.type}_general"

    def fetch(self):
        self.access() or self.refresh()
        self.s3.get() or self.s3.put()

        return self.s3.get()

    def get_url(self, file_key=None):
        return f"{self.s3.s3_settings.ENDPOINT_URL}/{self.s3.s3_settings.BUCKET_NAME}/{file_key or self.file_key}"

    @property
    def file_key(self):
        return f"{self.parent.sku}.{self.id}.{self.format}"

    def access(self):
        return requests.get(self.url).ok

    def exists(self):
        return self.s3.exists()

    @property
    def name(self):
        pattern1 = re.compile(r".*\/(?P<name>.*)")
        r = re.findall(pattern1, self.url)
        if not r or not r[0]:
            return None
        return r[0].split("?")[0]

    @property
    def format(self):
        pattern = re.compile(r"\/.*\.(\w+)(\?|$)")
        r = re.findall(pattern, self.url)
        return r[0][0] if r else None

    @backoff.on_exception(
        backoff.expo,
        (
            requests.ReadTimeout,
            requests.ConnectTimeout,
            requests.Timeout,
            requests.JSONDecodeError,
            requests.ConnectionError,
            requests.HTTPError,
        ),
        max_tries=8,
    )
    def request(self):
        response = requests.get(self.url)
        if not response.ok:
            raise MediaContentException
        response.raise_for_status()
        return response

    def get_content(func):
        def wrap(self, *args, **kwargs):
            if not self.response:
                if not self.access():
                    self.refresh()
                self.response = self.request()
            return func(self, *args, **kwargs)

        return wrap

    @property
    @get_content
    def content(self):
        return self.response.content

    @property
    @get_content
    def hash(self):
        return self.response.headers.get("x-amz-version-id")

    @property
    @get_content
    def mime(self):
        return self.response.headers.get("content-type")

    def get_badge_coordinates(self, badge):
        assert isinstance(badge, Page)
        # Если число отрицательное, посчитать позицию относительно координат базового изображения
        source_coordinates = [
            int(point.strip()) for point in badge.coordinates.split(",")
        ]
        plot = ImageRenderer(self.content)
        size = self.get_size(badge)
        result = []
        for i, s in enumerate(source_coordinates):
            point = s
            if s < 0:
                point = (plot.info["size"][i] + s) - size[i]
            result.append(point)

        return result

    def get_size(self, badge):
        assert isinstance(badge, Page)
        # Если указан знак %, то посчитать относительный размер
        if "%" in badge.size:
            percentage = int(re.sub(r"\W", "", badge.size))
            plot = ImageRenderer(self.content)
            result = [int(point / 100 * percentage) for point in plot.info["size"]]
        else:
            result = [int(point.strip()) for point in badge.size.split(",")]
        return result

    def apply_badges(self):
        if self.type not in ("image",):
            return self

        result = ImageRenderer(self.content)
        for badge in self.badges:
            if not requests.get(badge.file).ok:
                badge.refresh()

            _badge = ImageRenderer(badge.file)
            if badge.size:
                _badge.resize(self.get_size(badge))
            if badge.transparency:
                _badge.set_transparency(badge.transparency)
            result.paste(_badge, self.get_badge_coordinates(badge))

        # discount = ImageDraw.Draw(result)
        # myFont = ImageFont.truetype(
        #     str(GeneralSettings().TEMPLATES_PATH / 'BungeeSpice-Regular.ttf'.resolve()),
        #     72
        # )
        # discount.text((0, 0), "Sample text", font=myFont, fill=(255, 0, 0))
        return result


class MediaContentException(Exception):
    ...


class S3File:
    def __init__(self, url) -> None:
        self.response = None
        self.url = url

    def access(self):
        return requests.get(self.url).ok

    @property
    def format(self):
        pattern = re.compile(r"\/.*\.(\w+)(\?|$)")
        r = re.findall(pattern, self.url)
        return r[0][0] if r else None

    def request(self):
        response = requests.get(self.url)
        if not response.ok:
            raise MediaContentException
        return response

    def get_content(func):
        def wrap(self, *args, **kwargs):
            if not self.response:
                if not self.access():
                    raise NeedRefresh
                self.response = self.request()
            return func(self, *args, **kwargs)

        return wrap

    @property
    @get_content
    def content(self):
        return self.response.content

    @property
    @get_content
    def hash(self):
        return self.response.headers.get("x-amz-version-id")

    @property
    @get_content
    def mime(self):
        return self.response.headers.get("content-type")


class NeedRefresh(Exception):
    ...



from lxml import objectify

from gdshoplib.apps.platforms.base import Platform
from gdshoplib.packages.feed import Feed


class UlaManager(Platform):  # , Feed):
    DESCRIPTION_TEMPLATE = "ula.txt"
    KEY = "ULA"

    def get_shop(self):
        shop = objectify.Element("shop")
        objectify.deannotate(shop, cleanup_namespaces=True, xsi_nil=True)

        return shop

    def get_offer(self, product):
        offer = super(UlaManager, self).get_offer(product)
        offer.youlaCategoryId = self.feed_settings.ULA_CATEGORY_ID
        offer.youlaSubcategoryId = self.feed_settings.ULA_SUBCATEGORY_ID
        offer.tovary_vid_zhivotnogo = 10463
        offer.managerName = self.feed_settings.MANAGER_NAME
        offer.address = self.feed_settings.ADDRESS
        offer.phone = self.feed_settings.PHONE
        objectify.deannotate(offer, cleanup_namespaces=True, xsi_nil=True)
        return offer



from multiprocessing import Pool

import typer

from gdshoplib.apps.platforms import Platform, search_platform
from gdshoplib.apps.products.product import Product
from gdshoplib.services.notion.database import Database
from gdshoplib.services.notion.notion import Notion

app = typer.Typer()


@app.command()
def regenerate(single: bool = typer.Option(False)):
    if single:
        for product in Database(Product.SETTINGS.PRODUCT_DB).pages():
            if not product.sku:
                continue
            generate_description(product["id"])
    else:
        with Pool(3) as p:
            for product in Database(Product.SETTINGS.PRODUCT_DB).pages():
                p.apply_async(generate_description, (product["id"],))
            p.close()
            p.join()


@app.command()
def check(single: bool = typer.Option(False)):
    with Pool(3) as p:
        for product in Database(Product.SETTINGS.PRODUCT_DB).pages():
            p.apply_async(description_check_action, (product["id"],))
        p.close()
        p.join()


###


def generate_description(id):
    product = Product(id)
    product.description.warm_description_blocks()
    for platform, block in product.description.description_blocks.items():
        key = platform.split(":")[-1]

        new_description = product.description.generate(search_platform(key))
        Notion().update_block(
            block.id,
            params={"code": {"rich_text": [{"text": {"content": new_description}}]}},
        )
        print(f"{product.sku}: {platform}")


def description_check_action(id):
    product = Product(id)
    for platform_manager in Platform.__subclasses__():
        block = product.description.get_description_block(
            platform_key=platform_manager.KEY
        )
        print(f'{product.sku} {platform_manager}: {block.check if block else "None"}')



from gdshoplib.apps.platforms.base import Platform
from gdshoplib.apps.products.product import Product
from gdshoplib.cli.application import app as application

__all__ = ("Product", "Platform", "application")



from gdshoplib.services.notion.base import BasePage
from gdshoplib.services.notion.notion import Notion
from gdshoplib.services.notion.page import Page


class Database(BasePage):
    def get_source(self):
        return self.notion.get_database(self.id)

    def refresh(self):
        Notion(caching=True).get_database(self.id)
        self.initialize()

    def pages(self, filter=None, params=None):
        if not filter:
            for page in self.notion.get_pages(self.id, params=params):
                yield Page(page["id"], notion=self.notion, parent=self)
            return

        for page in self.notion.get_pages(self.id, params=params):
            filtered = True
            for k, v in filter.items():
                page = Page(page["id"], notion=self.notion, parent=self)
                if str(page.__getattr__(k)).lower() != str(v).lower():
                    filtered = False
            if filtered:
                yield page

    def commit(self):
        # Проитерироваться по изменениям и выполнить в Notion
        ...

    def to_json(self):
        # Вернуть товар в json
        ...



import typer

from gdshoplib.apps.crm.stats import Statistic

app = typer.Typer()


@app.command()
def price(price_type="average", filter_type="complited_shop"):
    if price_type == "average":
        print(Statistic().avg_price(filter_type=filter_type))

    elif price_type == "percentile":
        print(Statistic().percentile(filter_type=filter_type))



import re

from lxml import etree, objectify

from gdshoplib.apps.platforms.base import Platform
from gdshoplib.apps.products.product import Product
from gdshoplib.core.settings import ProductSettings
from gdshoplib.packages.feed import Feed
from gdshoplib.services.gdshop.gdshop import DB
from gdshoplib.services.notion.database import Database


class YandexMarketManager(Platform, Feed):
    KEY = "YAM"
    DESCRIPTION_TEMPLATE = "yam.txt"

    def get_categories(self):
        categories = Database(ProductSettings().CATEGORY_DB).pages()
        return [(str(i.uid), i.title) for i in categories]

    @staticmethod
    def fetch_id(id):
        return str(sum([int(i) for i in re.sub(r"\D*", "", id)]))

    @property
    def product_filter(self):
        return dict(status_description="Готово", status_publication="Публикация")

    @property
    def products(self):
        for sku in DB().get_market_products():
            if not sku:
                break
            product = Product.get(sku[0])
            if not product:
                break
            yield product

    def get_shop(self):
        shop = objectify.Element("shop")

        shop.name = self.feed_settings.SHOP_NAME
        shop.company = self.feed_settings.COMPANY_NAME
        shop.url = self.feed_settings.SHOP_URL

        currencies = objectify.Element("currencies")
        currency = etree.Element("currency")
        currency.attrib["id"] = "RUB"
        currency.attrib["rate"] = "1"
        objectify.deannotate(currency, cleanup_namespaces=True, xsi_nil=True)
        objectify.deannotate(currencies, cleanup_namespaces=True, xsi_nil=True)
        currencies.append(currency)
        shop.currencies = currencies

        categories = objectify.Element("categories")
        for ind, k in enumerate(
            ["Все товары", "Спорт и отдых", "Конный спорт"], start=1
        ):
            category = etree.Element("category")
            category.attrib["id"] = str(ind)
            if ind - 1 != 0:
                category.attrib["parentId"] = str(ind - 1)
            category.text = k
            categories.append(category)

        for _id, name in self.get_categories():
            category = etree.Element("category")
            category.attrib["id"] = _id
            category.attrib["parentId"] = "3"
            category.text = name
            objectify.deannotate(category, cleanup_namespaces=True, xsi_nil=True)
            categories.append(category)

        shop.categories = categories
        objectify.deannotate(shop, cleanup_namespaces=True, xsi_nil=True)

        return shop

    def get_offer(self, product):
        appt = objectify.Element("offer")
        appt.attrib["id"] = product.sku
        appt.count = product.quantity
        appt.weight = product.weight
        appt.dimensions = product.dimensions
        if product.quantity == 0:
            appt.attrib["type"] = "on.demand"

        appt.currencyId = "RUB"
        appt.price = round(product.price.now + (product.price.now * 0.19))
        if ((product.price.current_discount / product.price.base_price) * 100) > 5:
            appt.oldprice = round(
                product.price.base_price + (product.price.base_price * 0.19)
            )

        appt.purchase_price = product.price.neitral + (product.price.neitral * 0.19)
        appt.cofinance_price = product.price.neitral + (product.price.neitral * 0.19)

        for image in product.images:
            appt.addattr("picture", image.get_url())

        if product.color:
            appt.addattr("Цвет", product.color)
        if product.size:
            appt.addattr("Размер", product.size)

        appt.name = product.name
        appt.description = product.description.generate(self)
        appt.vendor = product.brand.title
        appt.categoryId = str(product.categories[0].uid)

        objectify.deannotate(appt, cleanup_namespaces=True, xsi_nil=True)
        return appt



from typing import List, Optional

from pydantic import BaseModel

from gdshoplib.services.vk.vk import VK


class VKStory(BaseModel):
    category: dict
    id: int
    description: Optional[str]
    price: dict
    title: str
    photos: List[dict]
    likes: dict
    reposts: dict
    views_count: int
    date: int
    sku: str
    raw: Optional[dict]

    @classmethod
    def parse_obj(cls, obj):
        result = super(VKStory, cls).parse_obj(obj)
        result.raw = obj
        return result


class VKNewStory(BaseModel):
    name: str
    description: str
    price: int
    old_price: Optional[int]
    main_photo_id: int
    photo_ids: Optional[List[Optional[int]]]
    sku: str


class VKEditProduct(BaseModel):
    name: str


class VKStories:
    def __init__(self, manager=None):
        self.manager = manager or VK()

    def list(self):
        result = self.manager.request(
            "market.get",
            params={
                "owner_id": f"-{self.manager.settings.VK_GROUP_ID}",
                "extended": 1,
                "with_disabled": 1,
                "need_variants": 1,
            },
        )["response"]
        if result:
            return [VKStory.parse_obj(item) for item in result["items"]]

    def get(self, sku):
        result = list(filter(lambda d: d.sku == sku, self.list()))
        if not result:
            return
        return result[0]

    def add(self, product: VKNewStory):
        assert isinstance(
            product, VKNewStory
        ), f"Продукт передан не того типа: {product.__class__} != VKNewStory"

        return self.manager.request(
            "market.add",
            params={
                "owner_id": f"-{self.manager.settings.VK_GROUP_ID}",
                "category_id": self.manager.settings.VK_CATEGORY_ID,
                **product.dict(),
            },
        )

    def edit(self, sku, product: VKEditProduct):
        assert isinstance(
            product, VKEditProduct
        ), f"Продукт передан не того типа: {product.__class__} != VKNewProduct"

        item = self.get(sku)
        return self.manager.request(
            "market.edit",
            params={
                "owner_id": f"-{self.manager.settings.VK_GROUP_ID}",
                "item_id": item.id,
                **product.dict(),
            },
        )

    def delete(self, sku):
        item = self.get(sku)
        return self.manager.request(
            "market.delete",
            params={
                "owner_id": f"-{self.manager.settings.VK_GROUP_ID}",
                "item_id": item.id,
            },
        )

    def album_get(self, sku):
        ...

    def album_create(self, sku):
        ...

    def album_edit(self, sku):
        ...

    def album_delete(self, sku):
        ...

    def album_add(self, sku):
        ...



import logging

from gdshoplib.services.vk.vk import VK

logger = logging.getLogger(__name__)


class VKMedia:
    def __init__(self, manager=None):
        self.manager = manager or VK()

    def get_upload_url(self):
        result = self.manager.request(
            "photos.getMarketUploadServer",
            params={"group_id": self.manager.settings.VK_GROUP_ID},
        )
        if result:
            return result["response"]["upload_url"]

    def _upload_photo(self, photo):
        return self.manager.request(
            None,
            params={},
            url=self.get_upload_url(),
            http_method="post",
            files={"file": (photo.name.split("/")[-1], photo)},
        )

    def upload(self, photo):
        upload_result = self._upload_photo(photo)
        result = self.manager.request(
            "photos.saveMarketPhoto",
            params={"group_id": self.manager.settings.VK_GROUP_ID, **upload_result},
        )
        if result:
            return result["response"][0]

    def delete(self, id):
        result = self.manager.request(
            "photos.delete",
            params={
                "owner_id": f"-{self.manager.settings.VK_GROUP_ID}",
                "photo_id": id,
            },
        )
        if result:
            return bool(result["response"] == 1)

    def get(self, id):
        # TODO: сделать работу с объектом
        result = self.manager.request(
            "photos.getById",
            params={
                "photos": f"-{self.manager.settings.VK_GROUP_ID}_{id}",
            },
        )
        if result.get("error"):
            if result["error"]["error_code"] == 200:
                return
            logger.warning(result)
            return
        else:
            return result["response"][0]



import time
from multiprocessing import Pool
from typing import Optional

import typer

from gdshoplib.apps.products.product import Product
from gdshoplib.packages.s3 import S3, SimpleS3Data
from gdshoplib.services.notion.database import Database

app = typer.Typer()


@app.command()
def warm(
    single: bool = typer.Option(False),
    with_badges: bool = typer.Option(False),
    loop_iteration: Optional[int] = typer.Option(None),
    sku: Optional[str] = typer.Option(None),
    all: bool = typer.Option(False),
):
    while True:
        gen = Product.query(filter=dict(status_publication="Публикация"))

        if sku:
            warm_product_media(Product.get(sku).id, with_badges)
            return

        if all:
            gen = Product.query()

        if single:
            for product in gen:
                warm_product_media(product["id"], with_badges)
        else:
            with Pool(7) as p:
                for product in gen:
                    p.apply_async(warm_product_media, (product["id"], with_badges))
                p.close()
                p.join()

        if loop_iteration:
            time.sleep(loop_iteration)
        else:
            break


@app.command()
def clean():
    s3 = S3(SimpleS3Data(None, file_key=None))
    s3.clean(pattern="Contents[? !contains(Key, `feed.`)][]")


@app.command()
def search(file_key=None):
    s3 = S3(SimpleS3Data(None, file_key=None))
    for object in s3.search(pattern=file_key):
        print(object)


@app.command()
def count():
    count = 0
    for product in Database(Product.SETTINGS.PRODUCT_DB).pages():
        count += len(Product(product["id"]).media)

    print(count)


@app.command()
def check(single: bool = typer.Option(False)):
    if single:
        for product in Database(Product.SETTINGS.PRODUCT_DB).pages():
            media_check_action(product["id"])
    else:
        with Pool(3) as p:
            for product in Database(Product.SETTINGS.PRODUCT_DB).pages():
                p.apply_async(media_check_action, (product["id"],))
            p.close()
            p.join()


###


def media_check_action(id):
    accepted_formats = (
        "png",
        "jpg",
        "jpeg",
    )
    product = Product(id)
    for media in Product(product.id).media:
        accepted = media.format in accepted_formats
        print(
            f"{media.file_key}: {media.exists()} {'ACCEPTED' if accepted else 'REJECTED'}"
        )


def warm_product_media(id, with_badges):
    for media in Product(id).images:
        media.fetch()
        print(f"{media.s3}: {media.exists()}")
        if with_badges and media.badges:
            render = media.apply_badges()
            badged = S3(
                SimpleS3Data(
                    render.content,
                    None,
                    render.info["mime"],
                    file_info={
                        "id": media.id,
                        "format": render.info["format"],
                        "prefix": "BADGED",
                    },
                    parent=media.parent,
                )
            )
            badged.put()
            print(f"{badged} {badged.exists()}")



import time
from typing import Optional

import typer

from gdshoplib.apps.crm.orders import Order

app = typer.Typer()


@app.command()
def on_request_update():
    # Взять все заказы, в нужном статусе
    # Обновить позиции на заказ
    for order in Order.query(params=Order.list_filter()):
        order.on_request.update()


@app.command()
def update(
    loop_iteration: Optional[int] = typer.Option(None),
    order_id: Optional[str] = typer.Option(None),
):
    # Обновить расчеты заказов
    if order_id:
        Order.update_one(order_id)
        return

    while True:
        Order.update()
        if loop_iteration:
            time.sleep(loop_iteration)
        else:
            break


@app.command()
def set_commission():
    ...


@app.command()
def update_decline(loop_iteration: Optional[int] = typer.Option(None)):
    # Обновить расчеты заказов
    while True:
        Order.update_decline()
        if loop_iteration:
            time.sleep(loop_iteration)
        else:
            break


@app.command()
def notify():
    # Разослать уведомление для менеджеров из CRM
    ...


if __name__ == "__main__":
    app()



from pydantic import BaseModel

from gdshoplib.apps.platforms.base import Platform
from gdshoplib.apps.products.product import Product
from gdshoplib.packages.feed import Feed
from gdshoplib.services.gdshop.gdshop import DB


class VKProductModel(BaseModel):
    ...


class VKManager(Platform, Feed):
    DESCRIPTION_TEMPLATE = "vk.txt"
    KEY = "VK"

    @property
    def product_filter(self):
        return dict(status_description="Готово")

    # def get_products(self) -> List[VKProductModel]:
    #     ...

    # def push_feed(self):
    #     # Обновить товары в VK
    #     ...


class VKPartnersManager(Platform, Feed):
    DESCRIPTION_TEMPLATE = "vk.txt"
    KEY = "VK-PARTNERS"

    @property
    def products(self):
        for sku in DB().get_partners_products():
            if not sku:
                break
            product = Product.get(sku[0])
            if not product:
                break
            yield product



from typing import Optional

from pydantic import BaseModel, ValidationError

from gdshoplib.services.gdshop.gdshop import DB


class Cart:
    def __init__(self) -> None:
        self.raws = []

    @classmethod
    def get(cls, order_id):
        cart = Cart()
        for row in DB().get_cart(order_id):
            cart.append(row)

    def extend(self, product):
        try:
            self.raws.append(
                CartRow(
                    sku=product.sku,
                    name=product.name,
                    url="-",
                    supply_price=0,
                    sell_price=product.price_now,
                    quantity=1,
                    supply_discount="-",
                    error=None,
                )
            )
        except ValidationError as e:
            self.raws.append(
                CartRowInvalid(
                    sku=product.sku,
                    name=product.name,
                    url="-",
                    supply_price=0,
                    sell_price=product.price_now,
                    quantity=1,
                    supply_discount="-",
                    error=str(e),
                )
            )

    def append(self, row):
        keys = CartRow.schema()["properties"].keys()
        self.raws.append(CartRow(**dict(zip(keys, row))))

    def __contains__(self, product):
        return product.sku in [i.sku for i in self.raws]

    def update_cart_row(self):
        for r in self.rows:
            self.notion.update_block(
                r.block_id,
                params={
                    "table_row": {
                        "cells": [
                            [self.row_object_formater(r.name)],
                            [self.row_object_formater(r.link)],
                            [self.row_object_formater(r.price)],
                            [self.row_object_formater(r.quantity)],
                            [self.row_object_formater(r.error)],
                        ]
                    }
                },
            )

    def row_object_formater(self, data):
        return {
            "type": "text",
            "text": {"content": f"{data}" if data else "", "link": None},
            "annotations": {
                "bold": False,
                "italic": False,
                "strikethrough": False,
                "underline": False,
                "code": False,
                "color": "default",
            },
            "plain_text": f"{data}" if data else "",
            "href": None,
        }


class CartRow(BaseModel):
    sku: Optional[str]
    name: str
    url: str
    supply_price: float
    sell_price: Optional[str]
    quantity: int
    supply_discount: str
    error: Optional[str]


class CartRowInvalid(BaseModel):
    sku: Optional[str]
    name: Optional[str]
    url: Optional[str]
    supply_price: Optional[str]
    sell_price: Optional[str]
    quantity: Optional[str]
    supply_discount: Optional[str]
    error: str



import webbrowser
from typing import Optional

import qrcode
import qrcode.image.svg
import typer
from jinja2 import Environment, FileSystemLoader, select_autoescape

from gdshoplib.apps.products.product import Product
from gdshoplib.core.settings import GeneralSettings

app = typer.Typer()


class Content:
    DESCRIPTION_TEMPLATE = "cart.html"

    def __init__(self):
        self.settings = GeneralSettings()
        self.jinja2_env = self.jinja2_env()

    def generate(self, platform):
        return self.render(platform)

    def get_template(self):
        return self.jinja2_env.get_template(self.DESCRIPTION_TEMPLATE)

    def render(self, data):
        return self.get_template().render(**data)

    def jinja2_env(self):
        return Environment(
            loader=FileSystemLoader(self.settings.TEMPLATES_PATH),
            autoescape=select_autoescape(),
        )

    def save(self, data):
        filename = str((self.settings.TEMPLATES_PATH / "stock_cart.html").resolve())
        with open(filename, "w") as file:
            file.write(self.render(data))
        return filename

    def qr_code(self, content):
        file = str((self.settings.TEMPLATES_PATH / "qr.png").resolve())
        qr = qrcode.QRCode(border=0)
        qr.add_data(content)
        img = qr.make_image(fit=True)
        img.save(file)

        return file


def generate_cart(product_sku, count=1, size=None):
    product = Product.get(product_sku)
    # Сгенерировать текст для карточки
    for size in [size] if size else product.size.split("/"):
        for _ in range(count):
            content = Content()
            webbrowser.get("open -a /Applications/Google\ Chrome.app %s").open(
                content.save(
                    dict(
                        product=product,
                        size=size,
                        name=content.settings.TEMPLATES_PATH / "name.png",
                        qr=content.qr_code(product.url),
                    )
                )
            )
            input("Следующий")


@app.command()
def generate(
    sku: Optional[str] = typer.Option(None),
    count: int = 1,
    size: Optional[str] = typer.Option(None),
):
    """Генерация карточек"""
    # Получить продукт, пройтись по всем продуктам
    if sku:
        generate_cart(sku, count=count, size=size)
        return

    for product in Product.query(filter=dict(status_publication="Публикация")):
        generate_cart(product.sku)


if __name__ == "__main__":
    app()



import typer

from gdshoplib.cli.crm.order import app as order
from gdshoplib.cli.crm.stats import app as stats

app = typer.Typer()

app.add_typer(order, name="order")
app.add_typer(stats, name="stats")


if __name__ == "__main__":
    app()



from io import BufferedReader, BytesIO

import cv2
import numpy as np
from barcode import get_barcode_class
from barcode.writer import ImageWriter
from PIL.Image import Image
from pyzbar import pyzbar


class Barcode:
    def __init__(self, code, /, type="EAN13"):
        self.code = code
        self.barcode_type = type

    @classmethod
    def read(cls, file):
        if isinstance(file, BufferedReader) or isinstance(file, BytesIO):
            image = cv2.imdecode(np.asarray(bytearray(file.read())), cv2.IMREAD_COLOR)
        elif isinstance(file, bytes):
            image = cv2.imdecode(np.fromstring(file, np.uint8), cv2.IMREAD_COLOR)
        elif isinstance(file, Image):
            image = cv2.cvtColor(np.array(file), cv2.COLOR_RGB2BGR)
        else:
            assert file.exists(), "Переданный файл не существует"
            image = cv2.imread(str(file.resolve()))
        decoded_objects = pyzbar.decode(image)
        for obj in decoded_objects:
            return cls(int(obj.data.decode()), type=obj.type)

    def generate(self):
        result = BytesIO()
        barcode_class = get_barcode_class(self.barcode_type)
        image = barcode_class(str(self.code), writer=ImageWriter())
        image.render().save(result, format="PNG")
        return result.getvalue()



import datetime

from dateutil.parser import parse

import requests
from gdshoplib.packages.renderer import ImageRenderer
from gdshoplib.services.notion.base import BasePage
from gdshoplib.services.notion.block import Block
from gdshoplib.services.notion.models.props import PropModel
from gdshoplib.services.notion.notion import Notion


class Page(BasePage):
    def refresh(self):
        Notion(caching=True).get_page(self.id)
        self.initialize()

    def initialize(self):
        super(Page, self).initialize()
        self.properties = PageProperty(self.page)

    def blocks(self, filter=None):
        if not filter:
            for block in self.notion.get_blocks(self.id):
                yield Block(block["id"], notion=self.notion, parent=self)
            return

        for block in self.notion.get_blocks(self.id):
            for k, v in filter.items():
                block = Block(block["id"], notion=self.notion, parent=self)
                if str(block.__getattr__(k)).lower() == str(v).lower():
                    yield block

    def __getattr__(self, attr):
        try:
            return super(Page, self).__getattr__(attr)
        except PropGetexception:
            self.refresh()
            return super(Page, self).__getattr__(attr)

    def warm(self):
        self.refresh()
        for block in self.notion.get_blocks(self.id):
            Block(block["id"], parent=self).refresh()

    def commit(self):
        # Проитерироваться по изменениям и выполнить в Notion
        ...

    def to_json(self):
        # Вернуть товар в json
        ...


class PageProperty:
    def __init__(self, page):
        self.page = page

    def __getitem__(self, key):
        return self.__dict__.get(key) or self.__search_content(key)

    def __getattr__(self, key):
        return self[key]

    def __search_content(self, key):
        prop = self.get_model_by_key(key)
        return prop.get_data()

    def get_model_by_name(self, name):
        model = PropModel(name=name, page=self.page)
        fields = []
        for key, field_list in self.properties_keys_map.items():
            for field in field_list:
                if field["name"] == name:
                    fields.append(field)
        model.fields = fields
        return model

    def get_model_by_key(self, key):
        model = PropModel(key=key, page=self.page)
        model.fields = self.properties_keys_map.get(key)
        return model

    def get_model_by_id(self, id):
        model = PropModel(id=id, page=self.page)
        for key, field_list in self.properties_keys_map.items():
            for field in field_list:
                if field["id"] == id:
                    model.fields = [field]
        return model

    def __str__(self) -> str:
        return f"{self.__class__}"

    def __repr__(self) -> str:
        return f"{self.__class__}"

    def relation_handler(self, page_id_in_list):
        _id = page_id_in_list[0]["id"]
        return Page(_id)

    def relation_list_handler(self, page_ids):
        result = []
        for _id in page_ids:
            result.append(Page(_id["id"]))
        return result

    def date_handler(self, date):
        if not date:
            return
        elif isinstance(date, datetime.datetime):
            return date
        elif "start" in date:
            return parse(date["start"]).date()

        return parse(date).date()

    def image_handler(self, image):
        if not image:
            return

        response = requests.get(image[0]["file"]["url"])
        if not response.ok:
            raise PropGetexception

        return ImageRenderer(response.content)

    def files_field_handler(self, files):
        if not files:
            return None
        if len(files) == 1:
            return files[0]["file"]["url"]
        return [file["file"]["url"] for file in files]

    @property
    def properties_keys_map(self):
        return {
            "title": (
                dict(name="Name", id="title"),
                dict(name="Код", id="title"),
            ),
            "uid": (dict(name="uid", id="Mf%7DA"),),
            "error": (dict(name="Ошибка", id="OD%60%7C"),),
            "edited_by": (dict(name="Last edited by", id="~%7BrF"),),
            "work": (dict(name="Использовать", id="bAcI"),),
            "created_time": (
                dict(name="Created time", id="v%5Dsj", handler=self.date_handler),
            ),
            "short_description": (dict(name="Короткое описание", id="u_tU"),),
            "size": (
                dict(name="Размер", id="taW%3B"),
                dict(name="Размер", id="NUuD"),
            ),
            "notes_field": (dict(name="Примечания", id="sXND"),),
            "quantity": (dict(name="Кол-во", id="pXTy"),),
            "edited_time": (
                dict(name="Last edited time", id="mVEw", handler=self.date_handler),
                dict(name="Last edited time", id="B_xv", handler=self.date_handler),
            ),
            "collection": (dict(name="Коллекция", id="W%5BhI"),),
            "name": (dict(name="Название на русском", id="Tss%5D"),),
            "created_by": (dict(name="Created by", id="TbyK"),),
            "kit_field": (
                dict(
                    name="Комплект", id="QV%5D%5D", handler=self.relation_list_handler
                ),
            ),
            "tags_field": (dict(name="Теги", id="MqdC"),),
            "status_description": (dict(name="Описание", id="MUl%7C"),),
            "status_publication": (dict(name="Публикация", id="BeEA"),),
            "color": (dict(name="Цвет", id="Jvku"),),
            "weight": (dict(name="Вес", id="Q%3FVy"),),
            "dimensions": (dict(name="Габариты", id="Q~eD"),),
            "specifications_field": (
                dict(name="Материалы / Характеристики", id="COmf"),
            ),
            "barcode_image_field": (
                dict(name="Штрихкод", id="Yiu%7D", handler=self.image_handler),
            ),
            "barcode_code_field": (dict(name="Код штрихкода", id="Uzsu"),),
            "original_name": (dict(name="Оригинальное название", id="%5CiXC"),),
            "price_neitral": (dict(name="Безубыточность", id="VmWm"),),
            "price_now": (dict(name="Текущая Цена", id="Ddaz"),),
            "price_kit": (dict(name="Цена комплекта", id="Dwfs"),),
            "price_base": (dict(name="Базовая цена", id="%40AuV"),),
            "price_discount": (dict(name="Скидка", id="%5E%3AR%7B"),),
            "price_discount_pers": (dict(name="Скидка %", id="Vjs%7C"),),
            "price_current_discount": (dict(name="Текущая Скидка", id="syrp"),),
            "price_agent": (dict(name="Агентская Цена", id="vC%5E%3D"),),
            "price_agent_kit": (dict(name="Агентский комплект", id="M%60HY"),),
            "price_gross": (dict(name="Себестоимость", id="d%3DhO"),),
            "manager_commission": (dict(name="Комиссия", id="Fdtc"),),
            "avito_id": (dict(name="avito_id", id="voQ%3E"),),
            "vk_id": (dict(name="vk_id", id="Ve%7Ca"),),
            "discount_from_date": (
                dict(name="Дата поставки", id="%60_a%3D", handler=self.date_handler),
            ),
            "badge_field": (
                dict(name="🎖️ Бейджи", id="%3CU_H", handler=self.relation_list_handler),
            ),
            "file": (dict(name="Файл", id="uK%3CN", handler=self.files_field_handler),),
            "coordinates": (dict(name="Координаты", id="XU%7C%3F"),),
            "transparency": (dict(name="Прозрачность", id="RHj%5B"),),
            "sku": (dict(name="Наш SKU", id="BKOs"),),
            "sku_s": (dict(name="SKU поставщика", id="BHve"),),
            "price_eur": (dict(name="Цена (eur)", id="AyqD"),),
            "brand": (
                dict(name="Бренд", id="gk%40%3B", handler=self.relation_handler),
            ),
            "categories": (
                dict(name="Категории", id="%7CFzB", handler=self.relation_list_handler),
            ),
            "price_coefficient": (
                dict(name="Наценка", id="HjFs"),
                dict(name="Наценка", id="YsUp"),
                dict(name="Наценка", id="lNgd"),
            ),
            "order_type": (dict(name="Тип заказа", id="%40%5Eql"),),
            "status": (dict(name="Status", id="EGwa"),),
            "promocodes": (
                dict(
                    name="🎫 Промокоды",
                    id="L%5CU%60",
                    handler=self.relation_list_handler,
                ),
            ),
            "pay_url": (dict(name="Оплата", id="%5D%7BRo"),),
            "price_result": (dict(name="Итого", id="bAsK"),),
            "products_field": (
                dict(name="🥇 Товары", id="e%3BIp", handler=self.relation_list_handler),
            ),
            "customer": (
                dict(name="Клиент", id="hGXc", handler=self.relation_handler),
            ),
            "discount": (
                dict(name="Скидка", id="k%5EOl"),
                dict(name="Скидка", id="vI~q"),
                dict(name="Персональная скидка", id="iXkC"),
            ),
            "platform": (
                dict(name="Откуда", id="n_pE", handler=self.relation_handler),
            ),
            "nps": (dict(name="NPS", id="%7Biv%7B", handler=self.relation_handler),),
            "pk": (dict(name="ID заказа", id="%7CQ%5Dy"),),
            "profit_field": (dict(name="Прибыль", id="MaSy"),),
            "max_discount_field": (dict(name="max Скидка", id="%7DsC%3B"),),
            "key": (
                dict(
                    name="Ключ",
                    id="P%7CTt",
                ),
            ),
            "rejection_reason": (
                dict(
                    name="Причина отказа",
                    id="P%7CTt",
                ),
            ),
        }


class PropGetexception(Exception):
    ...



from jinja2 import Environment, FileSystemLoader, select_autoescape

from gdshoplib.core.settings import GeneralSettings
from gdshoplib.services.notion.block import Block


class ProductDescription:
    def __init__(self, product):
        self.product = product
        self.general_settings = GeneralSettings()
        self.description_blocks = {}
        self.jinja2_env = self.jinja2_env()

    def warm_description_blocks(self):
        for block in self.product.blocks(filter={"type": "code"}):
            capture = block.get_capture()
            if "description" in capture:
                self.description_blocks[capture] = ProductDescriptionBlock(
                    block.id,
                    notion=self.product.notion,
                    parent=self.product.parent,
                )

    def get_description_block(self, platform_key=None):
        if not self.description_blocks:
            self.warm_description_blocks()

        if platform_key:
            return self.description_blocks.get(f"description:{platform_key.lower()}")
        return self.description_blocks.get("description")

    def generate(self, platform):
        return self.render(platform)

    def get_template(self, platform):
        return self.jinja2_env.get_template(platform.DESCRIPTION_TEMPLATE)

    def render(self, platform):
        return self.get_template(platform).render(product=self.product)

    def jinja2_env(self):
        return Environment(
            loader=FileSystemLoader(self.general_settings.TEMPLATES_PATH),
            autoescape=select_autoescape(),
        )


class ProductDescriptionBlock(Block):
    def __init__(self, *args, **kwargs):
        super(ProductDescriptionBlock, self).__init__(*args, **kwargs)

    @property
    def content(self):
        return self.code["rich_text"][0]["plain_text"]

    @property
    def check(self):
        return bool(self.code["rich_text"][0]["plain_text"])



import typer

from gdshoplib.cli.service.avito import app as avito
from gdshoplib.cli.service.vk import app as vk

app = typer.Typer()

app.add_typer(vk, name="vk")
app.add_typer(avito, name="avito")

if __name__ == "__main__":
    app()



import datetime
import time
from typing import Optional

from pydantic import BaseModel

from gdshoplib.apps.crm.cart import Cart
from gdshoplib.apps.crm.on_request import (OnRequestTable,
                                           OnRequestTableInvalidRow)
from gdshoplib.apps.products.price import Price
from gdshoplib.apps.products.product import Product
from gdshoplib.core.settings import CRMSettings
from gdshoplib.services.gdshop.gdshop import DB
from gdshoplib.services.notion.database import Database
from gdshoplib.services.notion.notion import Notion
from gdshoplib.services.notion.page import Page


class Order(Page):
    SETTINGS = CRMSettings()

    def __init__(self, *args, **kwargs):
        self._on_request = None
        self._products = None

        super(Order, self).__init__(*args, **kwargs)

    def __len__(self):
        return len(self.products) + sum([row.quantity for row in self.on_request])

    @staticmethod
    def list_filter(filter_type="raw"):
        if filter_type == "raw":
            return {
                "filter": {
                    "or": [
                        {"property": "Status", "status": {"equals": "Расчет"}},
                    ]
                }
            }
        elif filter_type == "complited_shop":
            return {
                "filter": {
                    "and": [
                        {"property": "Status", "status": {"equals": "Доставлено"}},
                        {"property": "Тип заказа", "select": {"equals": "Магазин"}},
                    ]
                }
            }
        elif filter_type == "complited_agent":
            return {
                "filter": {
                    "and": [
                        {"property": "Status", "status": {"equals": "Доставлено"}},
                        {"property": "Тип заказа", "select": {"equals": "Agent"}},
                    ]
                }
            }
        elif filter_type == "complited":
            return {
                "filter": {
                    "or": [
                        {
                            "property": "Status",
                            "status": {"does_not_equal": "Доставлено"},
                        },
                        {"property": "Status", "status": {"does_not_equal": "Отказ"}},
                        {
                            "property": "Status",
                            "status": {"does_not_equal": "Оформление"},
                        },
                        {"property": "Status", "status": {"does_not_equal": "Оплата"}},
                    ]
                }
            }
        elif filter_type == "actual":
            return {
                "filter": {
                    "or": [
                        {
                            "property": "Status",
                            "status": {"does_not_equal": "Комплектация"},
                        },
                        {
                            "property": "Status",
                            "status": {"does_not_equal": "Неразобранное"},
                        },
                        {"property": "Status", "status": {"does_not_equal": "Отказ"}},
                    ]
                }
            }
        elif filter_type == "waiting":
            return {
                "filter": {
                    "or": [
                        {"property": "Status", "status": {"equals": "Оплата"}},
                        {"property": "Status", "status": {"equals": "Комплектация"}},
                    ]
                }
            }
        elif filter_type == "new":
            return {
                "filter": {"property": "Status", "status": {"equals": "Неразобранное"}}
            }

    @classmethod
    def get(cls, id):
        page = [
            page
            for page in Database(cls.SETTINGS.CRM_DB).pages(
                params={
                    "filter": {
                        "property": "ID заказа",
                        "rich_text": {
                            "equals": id,
                        },
                    }
                }
            )
        ]
        if not page:
            return
        if len(page) > 1:
            raise OrderIDDuplicate

        page = page[0]

        return cls(page["id"], notion=page.notion, parent=page.parent)

    @classmethod
    def query(cls, filter=None, params=None, notion=None):
        for page in Database(cls.SETTINGS.CRM_DB, notion=notion).pages(
            filter=filter, params=params
        ):
            yield cls(page["id"], notion=page.notion, parent=page.parent)

    @property
    def on_request(self):
        if not self._on_request:
            self._on_request = OnRequestTable(
                [block for block in self.blocks(filter={"type": "table_row"})],
                notion=self.notion,
                parent=self,
            )

        return self._on_request

    @property
    def products(self):
        if not self._products:
            self._products = [Product(page.id) for page in self.products_field]
        return self._products

    def requested_price(self, product):
        return product.price.profit - product.price.profit * (self.discount * 0.01)

    @property
    def delivery(self):
        return

    @property
    def pay(self):
        return

    @property
    def discount_sum(self):
        return round(self.full_price - self.price, 2)

    @property
    def max_discount(self):
        # Расчет максимальной скидки
        if self.order_type != "Магазин":
            return 0

        now = self.on_request.price()
        for product in self.products:
            now += product.price.now
        lowest = self.on_request.price()
        for product in self.products:
            lowest += product.price.neitral

        result = now - lowest
        if not result:
            return 0
        return round((result) / (now * 0.01))

    def get_promocode(self):
        if not self.promocodes or self.order_type != "Магазин":
            return None
        self.promocodes.sort(key=lambda x: x.discount)
        return self.promocodes[0]

    @property
    def price(self):
        result = self.on_request.price()
        for product in self.products:
            result += product.price.now
        return result
        result = self.on_request.price()
        for product in self.products:
            if self.order_type == "Агент":
                result += product.price.neitral
            else:
                promocode = self.get_promocode()
                if promocode and product.price.current_discount < promocode.discount:
                    _price = (
                        product.price.profit
                        if product.price.profit > product.price.now
                        else product.price.now
                    )
                    result += _price - _price * (promocode.discount * 0.01)
                else:
                    result += product.price.now

        return round(result - result * (self.discount * 0.01))

    @property
    def full_price(self):
        result = self.on_request.price()
        for product in self.products:
            result += (
                product.price.profit
                if product.price.profit > product.price.now
                else product.price.now
            )

        return result

    @property
    def profit(self):
        profit = self.on_request.price("profit")
        for product in self.products:
            profit += product.price.profit
        return profit

    @property
    def commission(self):
        commission = self.on_request.price("profit") * 0.3
        for product in self.products:
            commission += product.price.profit * 0.15
        return round(commission)

    def generate_id(self):
        return f"{self.platform.key.lower()}.{int(time.time())}"

    def description_on_request_product(self, row):
        quantity = f"({row.quantity} шт.)"
        return f"{row.name} {quantity if row.quantity > 1 else ''}: {Price(row)['neitral'] * row.quantity} ₽"

    def description_basic_product(self, product):
        _price = (
            product.price.profit
            if product.price.profit > product.price.now
            else product.price.now
        )
        return f"{product.name}: {_price} ₽"

    @property
    def description(self):
        result = ["Ваш заказ\n"]
        counter = 1

        for row in self.on_request:
            if isinstance(row, OnRequestTableInvalidRow):
                continue
            result.append(f"{counter}. {self.description_on_request_product(row)}")
            counter += 1

        for product in self.products:
            result.append(f"{counter}. {self.description_basic_product(product)}")
            counter += 1

        if self.delivery or self.discount_sum:
            result.append(f"\nТовары ({len(self)}): {self.full_price}")

        if self.get_promocode():
            promocode = self.get_promocode()
            result.append(f"Промокод: {promocode.title} (-{promocode.discount}%)")

        if self.discount_sum:
            result.append(f"Скидка: {self.discount_sum}")

        if self.delivery:
            result.append(f"Стоимость доставки: {self.delivery.price}")

        result.append(f"\nОбщая стоимость: {self.price}")

        return "\n".join(result)

    def set_id(self):
        # Установить ID
        if not self.platform:
            return

        self.notion.update_prop(
            self.id,
            params={
                "properties": {"ID заказа": [{"text": {"content": self.generate_id()}}]}
            },
        )

    def set_max_discount(self):
        # Установка максимальной скидки
        self.notion.update_prop(
            self.id,
            params={"properties": {"max Скидка": {"number": self.max_discount or 0}}},
        )

    def set_price_description(self):
        # Установка описания расчетов цены
        self.notion.update_prop(
            self.id,
            params={
                "properties": {
                    "Расчеты": [{"text": {"content": self.description or ""}}]
                }
            },
        )

    def set_price(self):
        # Установка итоговой цены
        self.notion.update_prop(
            self.id,
            params={"properties": {"Итого": {"number": self.price}}},
        )

    def set_profit(self):
        # Установка итоговой цены
        self.notion.update_prop(
            self.id,
            params={"properties": {"Прибыль": {"number": self.profit}}},
        )

    def set_commission(self):
        # Установка комиссии менеджера
        self.notion.update_prop(
            self.id,
            params={"properties": {"Комиссия": {"number": self.commission}}},
        )

    def update_status(self, status):
        # Установка итоговой цены
        self.notion.update_prop(
            self.id,
            params={"properties": {"Status": {"status": {"name": status}}}},
        )

    def load_tasks(self):
        # Загрузка задач в CRM из источников
        ...

    def get_cart(self):
        # Получить список строк из базы
        return Cart.get(self.id)

    def update_cart(self):
        # Получить список всех товаров в карточке
        # Получить таблицу в расчетами
        cart = self.get_cart()
        for product in self.products:
            if product not in cart:
                cart.extend(product)
        # Добавить элементы карточки в таблицу
        # Обновить таблицу целиком
        ...

    @classmethod
    def update(cls):
        # Обновление карточки Order
        for order in cls.query(
            notion=Notion(caching=True), params=cls.list_filter(filter_type="raw")
        ):
            order.set_profit()
            order.set_price()
            order.set_commission()

            order.update_status("Комплектация")
            if not order.pk:
                order.set_id()

    @classmethod
    def update_one(cls, id):
        order = Order.get(id)
        order.set_profit()
        order.set_price()
        order.set_commission()

        order.update_status("Комплектация")

    @classmethod
    def update_decline(cls):
        date = datetime.date.today() - datetime.timedelta(days=7)
        for order in cls.query(
            notion=Notion(caching=True), params=cls.list_filter(filter_type="waiting")
        ):
            if order.edited_time.date() < date:
                order.update_status("Отказ")

    def notification(self):
        # Отправка уведомлений
        ...


class OrderIDDuplicate(Exception):
    ...



from datetime import timedelta

from temporalio import workflow

# Import activity, passing it through the sandbox without reloading the module
with workflow.unsafe.imports_passed_through():
    from gdshoplib.activities.notion import say_hello


@workflow.defn
class SayHello:
    @workflow.run
    async def run(self, name: str) -> str:
        return await workflow.execute_activity(
            say_hello, name, start_to_close_timeout=timedelta(seconds=5)
        )



class Delivery:
    # Расчет веса
    # Расчет упаковки
    # Расчет цены
    # Печать накладной
    # Выбор провайдера
    # Установка ID
    # Возврат информации о доставке
    # Hook для статуса
    ...
